@@ -1,16 +1,16 @@
 use crate::value::to_value;
-use crate::{EvaError, EvaItemKind, EvaResult, Value, OID};
+use crate::{is_str_any, is_str_wildcard, EResult, Error, ItemKind, Value, OID};
 use serde::{ser::SerializeSeq, Deserialize, Deserializer, Serialize, Serializer};
 use std::cmp::Ordering;
 use std::collections::HashSet;
 use std::convert::TryFrom;
 use std::fmt;
 use std::hash::{Hash, Hasher};
-use std::iter::FromIterator;
 use std::str::FromStr;
 
 static ERR_INVALID_OID_MASK: &str = "Invalid OID mask format";
 static ERR_PATH_MASK_EMPTY: &str = "Empty path mask";
+static ERR_INVALID_OID_MASK_OP: &str = "Invalid OID mask for this op";
 
 #[derive(Debug, Clone, Eq)]
 pub struct PathMask {
@@ -23,14 +23,6 @@ impl PathMask {
         Self { chunks: None }
     }
     #[inline]
-    fn is_str_any(s: &str) -> bool {
-        s == "#" || s == "*"
-    }
-    #[inline]
-    fn is_str_wildcard(s: &str) -> bool {
-        s == "+" || s == "?"
-    }
-    #[inline]
     fn is_any(&self) -> bool {
         self.chunks.is_none()
     }
@@ -40,10 +32,10 @@ impl PathMask {
             loop {
                 if let Some(i_chunk) = path_split.next() {
                     if let Some(m_chunk) = s_m.next() {
-                        if PathMask::is_str_any(m_chunk) {
+                        if is_str_wildcard(m_chunk) {
                             return true;
                         }
-                        if !PathMask::is_str_wildcard(m_chunk) && i_chunk != m_chunk {
+                        if !is_str_any(m_chunk) && i_chunk != m_chunk {
                             return false;
                         }
                     } else {
@@ -92,19 +84,11 @@ impl<'de> serde::de::Visitor<'de> for PathMaskVisitor {
     }
 }
 
-#[derive(Debug, Clone, Eq, PartialEq)]
+#[derive(Debug, Clone, Eq, PartialEq, Default)]
 pub struct PathMaskList {
     pub path_masks: HashSet<PathMask>,
 }
 
-impl Default for PathMaskList {
-    fn default() -> Self {
-        Self {
-            path_masks: <_>::default(),
-        }
-    }
-}
-
 impl Serialize for PathMaskList {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
@@ -135,18 +119,20 @@ impl From<PathMaskList> for Value {
 }
 
 impl TryFrom<Value> for PathMaskList {
-    type Error = EvaError;
-    fn try_from(value: Value) -> EvaResult<PathMaskList> {
+    type Error = Error;
+    fn try_from(value: Value) -> EResult<PathMaskList> {
         match value {
             Value::Seq(_) => Ok(PathMaskList::from_string_vec(&value.deserialize_into()?)?),
             Value::String(s) => {
                 if s.is_empty() {
                     Ok(<_>::default())
                 } else {
-                    Ok(PathMaskList::from_str_list(&Vec::from_iter(s.split(',')))?)
+                    Ok(PathMaskList::from_str_list(
+                        &s.split(',').collect::<Vec<_>>(),
+                    )?)
                 }
             }
-            _ => Err(EvaError::invalid_data("Expected vec or string")),
+            _ => Err(Error::invalid_data("Expected vec or string")),
         }
     }
 }
@@ -157,15 +143,15 @@ impl PathMaskList {
         Self { path_masks }
     }
 
-    pub fn from_str_list(s_masks: &[&str]) -> EvaResult<Self> {
+    pub fn from_str_list(s_masks: &[&str]) -> EResult<Self> {
         let mut path_masks = HashSet::new();
         for s in s_masks {
             path_masks.insert(s.parse()?);
         }
         Ok(Self { path_masks })
     }
-
-    pub fn from_string_vec(s_masks: &Vec<String>) -> EvaResult<Self> {
+    #[allow(clippy::ptr_arg)]
+    pub fn from_string_vec(s_masks: &Vec<String>) -> EResult<Self> {
         let mut path_masks = HashSet::new();
         for s in s_masks {
             path_masks.insert(s.parse()?);
@@ -226,16 +212,16 @@ impl fmt::Display for PathMask {
 }
 
 impl FromStr for PathMask {
-    type Err = EvaError;
+    type Err = Error;
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         if s.is_empty() {
-            Err(EvaError::invalid_data(ERR_PATH_MASK_EMPTY))
-        } else if PathMask::is_str_any(s) {
+            Err(Error::invalid_data(ERR_PATH_MASK_EMPTY))
+        } else if is_str_wildcard(s) {
             Ok(Self::new_any())
         } else {
             let mut chunks = Vec::new();
             for chunk in s.split('/') {
-                if PathMask::is_str_any(chunk) {
+                if is_str_wildcard(chunk) {
                     chunks.push("#".to_owned());
                     break;
                 }
@@ -248,17 +234,9 @@ impl FromStr for PathMask {
     }
 }
 
-#[derive(Debug, Clone, Eq, PartialEq)]
+#[derive(Debug, Clone, Eq, PartialEq, Default)]
 pub struct OIDMaskList {
-    pub oid_masks: HashSet<OIDMask>,
-}
-
-impl Default for OIDMaskList {
-    fn default() -> Self {
-        Self {
-            oid_masks: <_>::default(),
-        }
-    }
+    oid_masks: HashSet<OIDMask>,
 }
 
 impl Serialize for OIDMaskList {
@@ -289,14 +267,25 @@ impl OIDMaskList {
     pub fn new(oid_masks: HashSet<OIDMask>) -> Self {
         Self { oid_masks }
     }
-    pub fn from_str_list(s_masks: &[&str]) -> EvaResult<Self> {
+    #[inline]
+    fn new0(mask: OIDMask) -> Self {
+        let mut oid_masks = HashSet::new();
+        oid_masks.insert(mask);
+        Self { oid_masks }
+    }
+    #[inline]
+    pub fn new_any() -> Self {
+        Self::new0(OIDMask::new_any())
+    }
+    pub fn from_str_list(s_masks: &[&str]) -> EResult<Self> {
         let mut oid_masks = HashSet::new();
         for s in s_masks {
             oid_masks.insert(s.parse()?);
         }
         Ok(Self { oid_masks })
     }
-    pub fn from_string_vec(s_masks: &Vec<String>) -> EvaResult<Self> {
+    #[allow(clippy::ptr_arg)]
+    pub fn from_string_vec(s_masks: &Vec<String>) -> EResult<Self> {
         let mut oid_masks = HashSet::new();
         for s in s_masks {
             oid_masks.insert(s.parse()?);
@@ -322,25 +311,83 @@ impl OIDMaskList {
     pub fn is_empty(&self) -> bool {
         self.oid_masks.is_empty()
     }
+    pub fn as_string_vec(&self) -> Vec<String> {
+        self.oid_masks.iter().map(ToString::to_string).collect()
+    }
+    pub fn oid_masks(&self) -> &HashSet<OIDMask> {
+        &self.oid_masks
+    }
 }
 
 #[derive(Debug, Clone, Eq)]
 pub struct OIDMask {
-    kind: Option<EvaItemKind>,
+    kind: Option<ItemKind>,
     path: PathMask,
 }
 
 impl OIDMask {
     #[inline]
-    pub fn kind(&self) -> Option<EvaItemKind> {
+    fn check(s: &str) -> EResult<()> {
+        if s.len() > 65000 {
+            return Err(Error::invalid_data("OID mask too long"));
+        }
+        for c in s.chars() {
+            if !(c.is_alphanumeric() || crate::OID_MASK_ALLOWED_SYMBOLS.contains(c) || c == '/') {
+                return Err(Error::invalid_data(format!(
+                    "Invalid symbol in OID mask: {}",
+                    c
+                )));
+            }
+        }
+        Ok(())
+    }
+    #[inline]
+    pub fn kind(&self) -> Option<ItemKind> {
         self.kind
     }
+    /// A special case, when OID mask can be converted to "wildcard OID" - an OID, where id is the
+    /// wildcard symbol. Wildcard OIDs are special types of OIDs, which are fully compatible with
+    /// majority of ACL checkers and can be used to obtain data from various database sources,
+    /// which support wildcard selections (such as like 'kind:group/%' in SQL
+    #[inline]
+    pub fn to_wildcard_oid(&self) -> EResult<OID> {
+        if let Some(kind) = self.kind {
+            if let Some(ref ch) = self.path.chunks {
+                for (i, p) in ch.iter().enumerate() {
+                    if is_str_any(p) || (is_str_wildcard(p) && i < p.len()) {
+                        return Err(Error::invalid_data(ERR_INVALID_OID_MASK_OP));
+                    }
+                }
+            }
+            Ok(OID::new0_unchecked(kind, &self.path.to_string())?)
+        } else {
+            Err(Error::invalid_data(ERR_INVALID_OID_MASK_OP))
+        }
+    }
+    #[inline]
+    pub fn to_path(&self) -> String {
+        if self.path.chunks.is_some() {
+            format!(
+                "{}/{}",
+                if let Some(ref kind) = self.kind {
+                    kind.as_str()
+                } else {
+                    "+"
+                },
+                self.path
+            )
+        } else if let Some(ref kind) = self.kind {
+            format!("{}/#", kind.as_str())
+        } else {
+            "#".to_owned()
+        }
+    }
     #[inline]
     pub fn chunks(&self) -> Option<Vec<&str>> {
         self.path
             .chunks
             .as_ref()
-            .map(|v| v.iter().map(|x| x.as_str()).collect())
+            .map(|v| v.iter().map(String::as_str).collect())
     }
     #[inline]
     pub fn new_any() -> Self {
@@ -401,15 +448,25 @@ impl From<OID> for OIDMask {
     }
 }
 
+impl From<OID> for OIDMaskList {
+    fn from(oid: OID) -> Self {
+        let mask = OIDMask {
+            kind: Some(oid.kind()),
+            path: oid.full_id().parse().unwrap(),
+        };
+        mask.into()
+    }
+}
+
 impl FromStr for OIDMask {
-    type Err = EvaError;
+    type Err = Error;
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        if PathMask::is_str_any(s) {
+        if is_str_wildcard(s) {
             Ok(Self::new_any())
         } else {
             s.find(':').map_or_else(
                 || {
-                    let kind: EvaItemKind = s.parse()?;
+                    let kind: ItemKind = s.parse()?;
                     Ok(OIDMask {
                         kind: Some(kind),
                         path: PathMask::new_any(),
@@ -417,20 +474,22 @@ impl FromStr for OIDMask {
                 },
                 |tpos| {
                     if tpos == s.len() {
-                        Err(EvaError::invalid_data(format!(
+                        Err(Error::invalid_data(format!(
                             "{}: {}",
                             ERR_INVALID_OID_MASK, s
                         )))
                     } else {
                         let tp_str = &s[..tpos];
-                        let kind: Option<EvaItemKind> = if PathMask::is_str_wildcard(tp_str) {
+                        let kind: Option<ItemKind> = if is_str_any(tp_str) {
                             None
                         } else {
                             Some(s[..tpos].parse()?)
                         };
+                        let p = &s[tpos + 1..];
+                        OIDMask::check(p)?;
                         Ok(OIDMask {
                             kind,
-                            path: s[tpos + 1..].parse()?,
+                            path: p.parse()?,
                         })
                     }
                 },
@@ -442,7 +501,7 @@ impl FromStr for OIDMask {
 impl Hash for OIDMask {
     fn hash<H: Hasher>(&self, hasher: &mut H) {
         self.kind.map_or(0, |v| v as u16).hash(hasher);
-        self.path.hash(hasher)
+        self.path.hash(hasher);
     }
 }
 
@@ -462,33 +521,192 @@ impl<'de> Deserialize<'de> for OIDMask {
     }
 }
 
+impl Serialize for OIDMask {
+    #[inline]
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_str(&self.to_string())
+    }
+}
+
 impl From<OIDMaskList> for Value {
     fn from(v: OIDMaskList) -> Value {
         to_value(v).unwrap()
     }
 }
 
+impl From<OIDMask> for OIDMaskList {
+    fn from(mask: OIDMask) -> Self {
+        OIDMaskList::new0(mask)
+    }
+}
+
 impl TryFrom<Value> for OIDMaskList {
-    type Error = EvaError;
-    fn try_from(value: Value) -> EvaResult<OIDMaskList> {
+    type Error = Error;
+    fn try_from(value: Value) -> EResult<OIDMaskList> {
         match value {
             Value::Seq(_) => Ok(OIDMaskList::from_string_vec(&value.deserialize_into()?)?),
             Value::String(s) => {
                 if s.is_empty() {
                     Ok(<_>::default())
                 } else {
-                    Ok(OIDMaskList::from_str_list(&Vec::from_iter(s.split(',')))?)
+                    Ok(OIDMaskList::from_str_list(
+                        &s.split(',').collect::<Vec<_>>(),
+                    )?)
                 }
             }
-            _ => Err(EvaError::invalid_data("Expected vec or string")),
+            _ => Err(Error::invalid_data("Expected vec or string")),
         }
     }
 }
 
+#[derive(Serialize, Deserialize, Eq, PartialEq, Hash, Copy, Clone, Debug)]
+#[serde(rename_all = "lowercase")]
+pub enum Op {
+    Log,
+    Supervisor,
+}
+
+impl fmt::Display for Op {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(
+            f,
+            "{}",
+            match self {
+                Op::Log => "log",
+                Op::Supervisor => "supervisor",
+            }
+        )
+    }
+}
+
+#[derive(Serialize, Deserialize, Default, Clone, Debug)]
+struct AclItemsPvt {
+    #[serde(default)]
+    items: OIDMaskList,
+    #[serde(default)]
+    pvt: PathMaskList,
+    #[serde(default)]
+    rpvt: PathMaskList,
+}
+
+#[derive(Serialize, Deserialize, Default, Clone, Debug)]
+struct AclItems {
+    #[serde(default)]
+    items: OIDMaskList,
+}
+
+#[allow(clippy::trivially_copy_pass_by_ref)]
+fn is_false(val: &bool) -> bool {
+    !val
+}
+
+/// The default ACL, used by most of services. Can be overriden with a custom one
+#[derive(Serialize, Deserialize, Clone, Debug)]
+pub struct Acl {
+    id: String,
+    #[serde(default, skip_serializing_if = "is_false")]
+    admin: bool,
+    #[serde(default)]
+    read: AclItemsPvt,
+    #[serde(default)]
+    write: AclItems,
+    #[serde(default)]
+    deny: AclItemsPvt,
+    #[serde(default)]
+    ops: HashSet<Op>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    meta: Option<Value>,
+    from: Vec<String>,
+}
+
+impl Acl {
+    #[inline]
+    pub fn id(&self) -> &str {
+        &self.id
+    }
+    pub fn get_items_allow_deny(&self) -> (Vec<String>, Vec<String>) {
+        if self.admin {
+            (vec!["#".to_owned()], vec![])
+        } else {
+            let mut allow: HashSet<String> = self.read.items.as_string_vec().into_iter().collect();
+            let allow_write: HashSet<String> =
+                self.write.items.as_string_vec().into_iter().collect();
+            allow.extend(allow_write);
+            let deny: HashSet<String> = self.deny.items.as_string_vec().into_iter().collect();
+            (allow.into_iter().collect(), deny.into_iter().collect())
+        }
+    }
+    #[inline]
+    pub fn check_admin(&self) -> bool {
+        self.admin
+    }
+    #[inline]
+    pub fn check_op(&self, op: Op) -> bool {
+        self.admin || self.ops.contains(&op)
+    }
+    #[inline]
+    pub fn check_item_read(&self, oid: &OID) -> bool {
+        self.admin
+            || ((self.read.items.matches(oid) || self.write.items.matches(oid))
+                && !self.deny.items.matches(oid))
+    }
+    #[inline]
+    pub fn check_item_write(&self, oid: &OID) -> bool {
+        self.admin || (self.write.items.matches(oid) && !self.deny.items.matches(oid))
+    }
+    #[inline]
+    pub fn check_pvt_read(&self, path: &str) -> bool {
+        self.admin || (self.read.pvt.matches(path) && !self.deny.pvt.matches(path))
+    }
+    #[inline]
+    pub fn require_admin(&self) -> EResult<()> {
+        if self.check_admin() {
+            Ok(())
+        } else {
+            Err(Error::access("admin access required"))
+        }
+    }
+    pub fn require_op(&self, op: Op) -> EResult<()> {
+        if self.check_op(op) {
+            Ok(())
+        } else {
+            Err(Error::access(format!("operation access required: {}", op)))
+        }
+    }
+    pub fn require_item_read(&self, oid: &OID) -> EResult<()> {
+        if self.check_item_read(oid) {
+            Ok(())
+        } else {
+            Err(Error::access(format!("read access required for: {}", oid)))
+        }
+    }
+    pub fn require_item_write(&self, oid: &OID) -> EResult<()> {
+        if self.check_item_write(oid) {
+            Ok(())
+        } else {
+            Err(Error::access(format!("write access required for: {}", oid)))
+        }
+    }
+    pub fn require_pvt_read(&self, path: &str) -> EResult<()> {
+        if self.check_pvt_read(path) {
+            Ok(())
+        } else {
+            Err(Error::access(format!("read access required for: {}", path)))
+        }
+    }
+    #[inline]
+    pub fn contains_acl(&self, acl_id: &str) -> bool {
+        self.from.iter().any(|v| v == acl_id)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::{OIDMask, OIDMaskList, PathMask, PathMaskList};
-    use crate::EvaItemKind;
+    use crate::{ItemKind, OID};
 
     #[test]
     fn test_path_mask() {
@@ -522,19 +740,22 @@ mod tests {
         let mask: OIDMask = s.parse().unwrap();
         assert_eq!(s, mask.to_string());
         assert_eq!(mask.path.chunks, None);
+        assert_eq!(mask.to_path(), "#");
         let s = "";
         assert_eq!(s.parse::<OIDMask>().is_err(), true);
         let s = "sensor:";
         assert_eq!(s.parse::<OIDMask>().is_err(), true);
         let s = "sensor:data/#";
         let mask: OIDMask = s.parse().unwrap();
+        assert_eq!(mask.to_path(), "sensor/data/#");
         assert_eq!(s, mask.to_string());
-        assert_eq!(mask.kind.unwrap(), EvaItemKind::Sensor);
+        assert_eq!(mask.kind.unwrap(), ItemKind::Sensor);
         assert_eq!(mask.path.chunks.unwrap(), ["data", "#"]);
         let s = "#:data/#";
         assert_eq!(s.parse::<OIDMask>().is_err(), true);
         let s = "+:data/tests/t1";
         let mask: OIDMask = s.parse().unwrap();
+        assert_eq!(mask.to_path(), "+/data/tests/t1");
         assert_eq!(s, mask.to_string());
         assert_eq!(mask.path.chunks.unwrap(), ["data", "tests", "t1"]);
         assert_eq!(mask.kind, None);
@@ -542,7 +763,7 @@ mod tests {
         let mask: OIDMask = s.parse().unwrap();
         assert_eq!(mask.to_string(), "unit:data/tests/#");
         assert_eq!(mask.path.chunks.unwrap(), ["data", "tests", "#"]);
-        assert_eq!(mask.kind.unwrap(), EvaItemKind::Unit);
+        assert_eq!(mask.kind.unwrap(), ItemKind::Unit);
         let s = "data/*/t1";
         let mask: PathMask = s.parse().unwrap();
         assert_ne!(s, mask.to_string());
@@ -635,4 +856,18 @@ mod tests {
         assert_eq!(p.matches(&"unit:a/b/zzz/xxx".parse().unwrap()), true);
         assert_eq!(p.matches(&"unit:a/c/zzz/xxx".parse().unwrap()), true);
     }
+
+    #[test]
+    fn test_oid_wildcard_mask() {
+        let mask: OIDMask = "sensor:tests/#".parse().unwrap();
+        let oid_mask: OID = mask.to_wildcard_oid().unwrap();
+        assert!(oid_mask.is_wildcard());
+        assert_eq!(oid_mask.to_wildcard_str("%"), "sensor:tests/%");
+        let mask: OIDMask = "sensor:#".parse().unwrap();
+        let oid_mask: OID = mask.to_wildcard_oid().unwrap();
+        assert!(oid_mask.is_wildcard());
+        assert_eq!(oid_mask.to_wildcard_str("%"), "sensor:%");
+        let mask: OIDMask = "sensor:+/#".parse().unwrap();
+        assert!(mask.to_wildcard_oid().is_err());
+    }
 }
