@@ -163,6 +163,32 @@ pub struct LocalStateEvent {
     pub t: f64,
 }
 
+#[derive(Debug, Clone, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
+pub struct RemoteStateEvent {
+    pub status: ItemStatus,
+    pub value: Value,
+    pub act: Option<usize>,
+    pub ieid: IEID,
+    pub t: f64,
+    pub node: String,
+    pub connected: bool,
+}
+
+impl RemoteStateEvent {
+    pub fn from_local_state_event(event: LocalStateEvent, system_name: &str) -> Self {
+        Self {
+            status: event.status,
+            value: event.value,
+            act: event.act,
+            ieid: event.ieid,
+            t: event.t,
+            node: system_name.to_owned(),
+            connected: true,
+        }
+    }
+}
+
 #[derive(Debug, Clone, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct ReplicationStateEvent {
