@@ -1,7 +1,7 @@
 use crate::registry;
 use crate::Value;
 use crate::{EResult, Error};
-use elbus::rpc::{RpcClient, RpcHandlers};
+use elbus::rpc::{self, RpcClient, RpcHandlers};
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
 use std::fmt;
@@ -31,12 +31,7 @@ impl Registry {
     }
     #[inline]
     pub async fn key_get(&self, key: &str) -> EResult<Value> {
-        registry::key_get(
-            &registry::format_svc_data_subkey(&self.id),
-            key,
-            &self.rpc,
-        )
-        .await
+        registry::key_get(&registry::format_svc_data_subkey(&self.id), key, &self.rpc).await
     }
     #[inline]
     pub async fn key_userdata_get(&self, key: &str) -> EResult<Value> {
@@ -44,49 +39,26 @@ impl Registry {
     }
     #[inline]
     pub async fn key_increment(&self, key: &str) -> EResult<i64> {
-        registry::key_increment(
-            &registry::format_svc_data_subkey(&self.id),
-            key,
-            &self.rpc,
-        )
-        .await
+        registry::key_increment(&registry::format_svc_data_subkey(&self.id), key, &self.rpc).await
     }
 
     #[inline]
     pub async fn key_decrement(&self, key: &str) -> EResult<i64> {
-        registry::key_decrement(
-            &registry::format_svc_data_subkey(&self.id),
-            key,
-            &self.rpc,
-        )
-        .await
+        registry::key_decrement(&registry::format_svc_data_subkey(&self.id), key, &self.rpc).await
     }
     #[inline]
     pub async fn key_get_recursive(&self, key: &str) -> EResult<Vec<(String, Value)>> {
-        registry::key_get_recursive(
-            &registry::format_svc_data_subkey(&self.id),
-            key,
-            &self.rpc,
-        )
-        .await
+        registry::key_get_recursive(&registry::format_svc_data_subkey(&self.id), key, &self.rpc)
+            .await
     }
     #[inline]
     pub async fn key_delete(&self, key: &str) -> EResult<Value> {
-        registry::key_delete(
-            &registry::format_svc_data_subkey(&self.id),
-            key,
-            &self.rpc,
-        )
-        .await
+        registry::key_delete(&registry::format_svc_data_subkey(&self.id), key, &self.rpc).await
     }
     #[inline]
     pub async fn key_delete_recursive(&self, key: &str) -> EResult<Value> {
-        registry::key_delete_recursive(
-            &registry::format_svc_data_subkey(&self.id),
-            key,
-            &self.rpc,
-        )
-        .await
+        registry::key_delete_recursive(&registry::format_svc_data_subkey(&self.id), key, &self.rpc)
+            .await
     }
 }
 
@@ -231,7 +203,27 @@ impl Initial {
     pub fn take_config(&mut self) -> Option<Value> {
         self.config.take()
     }
+    #[inline]
     pub async fn init_rpc<R>(&self, handlers: R) -> EResult<Arc<RpcClient>>
+    where
+        R: RpcHandlers + Send + Sync + 'static,
+    {
+        self.init_rpc_opts(handlers, rpc::Options::default()).await
+    }
+    #[inline]
+    pub async fn init_rpc_blocking<R>(&self, handlers: R) -> EResult<Arc<RpcClient>>
+    where
+        R: RpcHandlers + Send + Sync + 'static,
+    {
+        self.init_rpc_opts(
+            handlers,
+            rpc::Options::new()
+                .blocking_notifications()
+                .blocking_frames(),
+        )
+        .await
+    }
+    pub async fn init_rpc_opts<R>(&self, handlers: R, opts: rpc::Options) -> EResult<Arc<RpcClient>>
     where
         R: RpcHandlers + Send + Sync + 'static,
     {
@@ -240,7 +232,7 @@ impl Initial {
             elbus::ipc::Client::connect(&self.elbus_config()?),
         )
         .await??;
-        let rpc = RpcClient::new(bus, handlers);
+        let rpc = RpcClient::create(bus, handlers, opts);
         Ok(Arc::new(rpc))
     }
     pub fn init_registry(&self, rpc: &Arc<RpcClient>) -> Registry {
